import org.eclipse.jetty.server.*;
import org.eclipse.jetty.server.handler.*;
import org.eclipse.jetty.util.log.*;
import org.eclipse.jetty.servlet.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import webapps.ajax1.model.*;
import java.util.*;
import java.io.*;

/**
 * Objetivo: embeber servidor Jetty http://eclipse.org/jetty
 * 
 * Idea: permitir el desarrollo de aplicaciones web que permitan sofisticadas
 * interfases (jquery+jtable) de usuario, gastando los minimos recursos en el servidor. Facil
 * desarrollo para responder a peticiones via http, intercambio de mensajes
 * utilizando json
 * 
 * Documento Base: http://www.eclipse.org/jetty/documentation/current/embedding-jetty.html
 * 
 * Diseño:Jetty posee el concepto de handler para procesar peticiones via http, incluso es un
 * componente mas simple que un servlet para desarrollar y tiene funcionalidad similar.
 * Desarrollar handlers para cada iteraccion CRUD con tablas jtable
 * Utilizar jdbc para actualizar la base de datos, en este caso, jaybird para usar con Firebird SQL
 * Server.
 * Utilizar un Resource Handler para direccionar todo el contenido html estatico
 * Las paginas seran html, evitando realizar paginas jsp
 * El contenido dinamico se hara con jquery via mensajes json
 * De requerirse contenido dinamico, se podran hacer servlets para devolver paginas dinamicas al cliente
 * Usar Google Gson para procesar mensajes json del lado del servidor
 * 
 * Distribuir aplicacion con jars de jaybird+jetty
 * 
 * 1. bajando los jars que necesitamos http://www.eclipse.org/jetty/documentation/current/advanced-embedding.html
 * existe un jar que contiene todo + la api para servlets
 * http://repo1.maven.org/maven2/org/eclipse/jetty/aggregate/jetty-all/9.0.3.v20130506/
 *  jetty-all-9.0.3.v20130506.jar
 * http://repo1.maven.org/maven2/org/eclipse/jetty/orbit/javax.servlet/3.0.0.v201112011016/
 *  javax.servlet-3.0.0.v201112011016.jar
 * 2. agregando esos jars a las librerias del proyecto bluej, tools-pref-libraries-add, salir de bluej, volver a entrar
 * 3. para embeber jetty:
 * 
 *     1. Crear una instancia de Server
 *     2. Agregar/Configurar Connectors
 *     3. Agregar/Configurar Handlers y/o Contexts y/o Servlets
 *     4. Arrancar el Server
 *  hacer join para detener el thread actual hasta que termine la ejecucion del servidor o bien realizar alguna otra accion
 *  
 * 
 * este codigo requiere java 7, no funciona con jdk 6, las versiones anteriores de jetty
 * (que tal vez pueda compilarse con java 6) no incluyen clase ServerConnector, por lo tanto,
 * no es usable con jdk 6
 * para instalar jdk 7 en debian:
 
Instalacion Manual de java jdk descargado desde Oracle
fuente: http://blog.soluciones3f.com.ar/2013/01/18/oracle-java-jdk-7-en-debian-6-squeeze/
archivo jdk: jdk-7u25-linux-x64.tar.gz


# Descomprimirlo (estamos ubicados en el directorio donde pusimos la descarga)
tar -xvf jdk-7u25-linux-x64.tar.gz
# Crear directorio para todas las versiones de java
mkdir /usr/local/java
# Mover el directorio descomrpimido
mv jdk1.7.0_25 /usr/local/java
# Configurar Debian para que sepa donde pusimos Java
update-alternatives --install /usr/bin/java java /usr/local/java/jdk1.7.0_25/bin/java 1065
update-alternatives --install /usr/bin/javac javac /usr/local/java/jdk1.7.0_25/bin/javac 1065
update-alternatives --install /usr/bin/jar jar /usr/local/java/jdk1.7.0_25/bin/jar 1065
update-alternatives --install /usr/bin/javaws javaws /usr/local/java/jdk1.7.0_25/bin/javaws 1065
# Configuramos el default de java
update-alternatives --config java
# Si solo tenemos uno, el mensaje sería:
# -> There is only one alternative in link group java: /usr/local/java/jdk1.7.0_25/bin/java


#verificar la version instalada
java -version

# editar bash para que se cargue la variable JAVA_HOME
$ vi /etc/bash.bashrc
export JAVA_HOME=/usr/local/java/jdk1.7.0_25
export PATH=$PATH:$JAVA_HOME/bin

source /etc/bash.bashrc
source ~/.bashrc

BlueJ no reconoce la nueva version jdk 7 instalada manualmente, 
se requiere editar script de ejecucion de bluej:

 * 
 * 
#!/bin/sh

unset JAVAPATH
unset ARCH

if [ -x /usr/bin/apt-config ]; then
    # Ask apt-config for the architecture
    eval `/usr/bin/apt-config shell ARCH APT::Architecture`
elif [ -x /usr/bin/arch ]; then
    ARCH=`/usr/bin/arch`
fi

for VMNAME in java-7-oracle java-7-sun java-7-openjdk java-6-oracle java-6-sun java-6-openjdk ; do
    if [ -e /usr/local/java/$VMNAME/lib/tools.jar ]; then
#modificado manualmente grchere porque no reconoce jdk instalado
#manualmente, incluso, reinstalando nueva version de bluej!
        JAVAPATH="/usr/local/java/$VMNAME"
        break
    elif [ -e /usr/lib/jvm/$VMNAME/lib/tools.jar ]; then
        JAVAPATH="/usr/lib/jvm/$VMNAME"
        break
    elif [ -e "/usr/lib/jvm/$VMNAME-$ARCH/lib/tools.jar" ]; then
        JAVAPATH="/usr/lib/jvm/$VMNAME-$ARCH"
        break
    elif [ -e /usr/lib/jvm/$VMNAME-amd64/lib/tools.jar ]; then
        JAVAPATH="/usr/lib/jvm/$VMNAME-amd64"
        break
    elif [ -e /usr/lib/jvm/$VMNAME-i386/lib/tools.jar ]; then
        JAVAPATH="/usr/lib/jvm/$VMNAME-i386"
        break
    fi
done

if [ x"$JAVAPATH" = x"" ]; then
    echo "A suitable JDK couldn't be located. You may need to edit the"
    echo "/usr/bin/bluej launch script."
    exit 1
fi

echo $JAVAPATH  
CP="/usr/share/bluej/bluej.jar:$JAVAPATH/lib/tools.jar"
"$JAVAPATH/bin/java" -Dawt.useSystemAAFontSettings=on -cp "$CP" bluej.Boot  "$@"
 
 * 
 *
 *
 * 3. para embeber jetty:
 * 
 *     1. Crear una instancia de Server
 *     2. Agregar/Configurar Connectors
 *     3. Agregar/Configurar Handlers y/o Contexts y/o Servlets
 *     4. Arrancar el Server
 *  hacer join para detener el thread actual hasta que termine la ejecucion del servidor o bien realizar alguna otra accion
 *
 *
 * 
 * @author G.Cherencio
 * @version 1.0
 * @since 1.7
 */
public class JettyServer extends JFrame {
    private boolean isRunning = false;
    private JButton btn = null;
    private Server server;
    private ServerConnector http;
    private ServletContextHandler context;
    
    public JettyServer() {
System.out.println("Configuro server Jetty...");        
        server = new Server();
        http = new ServerConnector(server);
        http.setHost("localhost");
        //http.setHost("grcherepc.ddns.net");
        //http.setHost("200.200.200.102");
        http.setPort(8080);
        http.setIdleTimeout(30000);
        server.addConnector(http);
        
        // cargo handle generico, pero no lo utilizo (por ahora)
        Handler hellow = new handlers.TestHandler();
        
        // resource handle, para manejo de contenido html estatico
        ResourceHandler resource_handler = new ResourceHandler();
        resource_handler.setDirectoriesListed(true);
        resource_handler.setWelcomeFiles(new String[] { "index.html" });
        resource_handler.setResourceBase("./webapps");
        
        // configuracion servlets ajax1 application
        context = new ServletContextHandler(ServletContextHandler.SESSIONS);
        context.setContextPath("/ajax1"); // asume que los servlets son de la app ajax1
        context.setResourceBase("./webapps/ajax1/servlets"); // asume que las clases de servlets estan en el directorio actual      
        context.setClassLoader(Thread.currentThread().getContextClassLoader());
        // asocio servlets - uri  ajax1 application
        context.addServlet(webapps.ajax1.servlets.JsonServlet.class,            "/JsonServlet,/JsonServletDelete,/JsonServletUpdate,/JsonServletDates"); 
        context.addServlet(webapps.ajax1.servlets.JsonTestServlet.class,        "/listTest,/insertTest,/deleteTest,/updateTest");
        context.addServlet(webapps.ajax1.servlets.JsonQueryServlet.class,       "/query");
        context.addServlet(webapps.ajax1.servlets.LoginServlet.class,           "/login");
        context.addServlet(webapps.ajax1.servlets.SessionDataServlet.class,     "/session");
        context.addServlet(webapps.ajax1.servlets.SessionDatesServlet.class,    "/mesas");
        context.addServlet(webapps.ajax1.servlets.GridServlet.class,            "/grid");
        context.addServlet(webapps.ajax1.servlets.GraphServlet.class,           "/graph");
        
        // asocio lista de handlers al servidor (file server + servlets)
        HandlerList handlers = new HandlerList();
        // quite el handle hellow
        handlers.setHandlers(new Handler[] { context, resource_handler, new DefaultHandler() });
        server.setHandler(handlers);
        
        doStart();
        btn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if ( ((JButton) e.getSource()).getText().equalsIgnoreCase("Stop Server") ) {
                    doStop();
                } else {
                    doStart();
                }
            }
        });
        add(btn);
        addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                if ( isRunning ) doStop();
                // cerrar las conexiones a bases de datos!
                System.out.println("JettyServer, cierro conexiones!!");
                FBCon.closeAll();
                System.exit(0);
            }
        });
        setSize(300,300);
        setTitle("Jetty Server Control Panel");
        setVisible(true);
    }

    private boolean doStart() {
        try {
            server.start();
            isRunning=true;
            updateLabel("Stop Server");
            System.out.println("Servidor Started! version "+server.getVersion() + 
                " en puerto "+http.getPort());
            return true;
        } catch(Exception ex) {
            System.err.println("Error Starting Server ..."+ex.getMessage());
        }
        return false;
    }
    
    private boolean doStop() {
        try {
            server.stop();
            isRunning=false;
            updateLabel("Start Server");
            System.out.println("Servidor Stopped!");
            return true;
        } catch(Exception ex) {
            System.err.println("Error Stopping Server ..."+ex.getMessage());
        }
        return false;
    }
    
    private void updateLabel(final String label) {
        if ( btn == null ) btn = new JButton(label);
        else btn.setText(label);
    }
    
    public static void main(String[] args) throws Exception {
        Properties prop = new Properties();
        prop.load(new FileInputStream("jetty.prop"));
        String os=null;
        if (System.getProperty("os.name").equalsIgnoreCase("Linux")) {
            // asumo que ejecuta en Linux
            os="linux";
        } else {
            // asumo que ejecuta en Windows
            os="windows";
        }
        String host = prop.getProperty(os+".db.host");
        String port = prop.getProperty(os+".db.port");
        String database1 = prop.getProperty(os+".db.database1");
        String database2 = prop.getProperty(os+".db.database2");
        String user = prop.getProperty(os+".db.user");
        String password = prop.getProperty(os+".db.password");
        String role = prop.getProperty(os+".db.role");
System.out.println(host+"/"+port+":"+database1+" "+user+" "+password+" "+role);
System.out.println(host+"/"+port+":"+database2+" "+user+" "+password+" "+role);
        new FBCon(host+"/"+port+":"+database1,user,password,role);
        new FBCon(host+"/"+port+":"+database2,user,password,role);
        new JettyServer();
    }
}
