package webapps.ajax1.servlets;

import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.annotation.*;
import java.util.*;
import java.io.*;
import java.sql.*;
import webapps.ajax1.grid.*;
import webapps.ajax1.model.*;
/**
 * Servlet que da soporte a graficos Canvas JS (plugin jquery)
 * https://canvasjs.com
 * 
 * INPUT:
 *     parametro "name" nombre asociado a la serie de datos a devolver
 * OUTPUT: Devuelve arreglo asociativo que representa una serie de datos,
 * que sería un elemento a su vez de otro arreglo de nivel superior que es la 
 * propiedad data utilizada para realizar el grafico.
 * La propiedad data tiene la siguiente forma: (https://canvasjs.com/docs/charts/chart-options/data/)
 *
 *  ...
 *  data : {
 *      { data serie 1 },
 *      { data serie 2 },
 *      ...
 *      }
 *      
 *  este servlet no devuelve data, sino una data serie.
 *  Una propiedad data es un arreglo de dataPoints, este servlet devuelve
 *  un arreglo de tipo dataPoints
 *  
 *  El formato de un arreglo de data serie (dataPoints) es el siguiente:
 *  
 *  { 
 *      type: "column",
 *      dataPoints: [
 *         { y: <valor eje y>, x: <valor eje x>, label: "titulo eje x", 
 *             ... },
 *             ...
 *             ]
 *  }
 *  
 *  si el query da error, devuelve arreglo dataPoints vacio, sino lo carga
 *  con el resultado del query
 * 
 * @author G. Cherencio.
 * @version 1.0
 */
@WebServlet(name = "GridServlet", urlPatterns = {"/graph"})
public class GraphServlet extends HttpServlet {
    private static FBCon fb1 = FBCon.getFBCon(1);  // isft database

    /**
     * Processes requests for both HTTP
     * <code>GET</code> and
     * <code>POST</code> methods.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        response.setCharacterEncoding("UTF8");
        response.setContentType("application/json");
        response.setContentType("text/html");
        
        PrintWriter out = response.getWriter();
        String graphName = request.getParameter("name");
        StringBuffer js = new StringBuffer("{type:\"column\",dataPoints:[");
        if ( graphName.equalsIgnoreCase("alumnos_x_carrera") ) {
            // obtengo la cantidad de alumnos por carrera
            // el label del eje x es la descripcion corta de carrera
            try {
                ResultSet rs = fb1.getResultSet("SELECT P.SDESCR AS DESCR,COUNT(A.LEGAJOA) AS CNT "+
                    "FROM TBL_ALUMNO A INNER JOIN TBL_PLANE P ON A.IDPLAN = P.CODIGO "+
                    "GROUP BY P.SDESCR");
                boolean pasoPrimeraFila=false;
                while(rs.next()) {
                    if(pasoPrimeraFila) js.append(",");
                    if(!pasoPrimeraFila) pasoPrimeraFila=true;
                    js.append("{y:"+rs.getString("CNT")+",label:\""+
                        rs.getString("DESCR")+"\"}");
                }
                fb1.close(rs);
            } catch(java.sql.SQLException e) {
                // ignoro error, quedara el arreglo dataPoints[] vacio
            }            
        }
        js.append("]}");
        out.print(js.toString());
//TRACE        
System.out.println("graph servlet json="+js.toString()+"=");
        out.close();
    }
    
    /**
     * Handles the HTTP
     * <code>GET</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Handles the HTTP
     * <code>POST</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Returns a short description of the servlet.
     *
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>
}
